#ifndef SERVERWINDOW_H
#define SERVERWINDOW_H

#include <QMainWindow>
#include <src/Reconstruction/CloudFromDepth.h>
#include "src/ReactDomain.h"
#include "src/Cams/OpenCVCam/OpenCVCam.h"
#include "src/Cams/Freenect2/Freenect2Cam.h"
#include "src/Cams/FFmpegCam/FFmpegRecorder.h"
#include "src/Cams/FFmpegCam/FFmpegPlayer.h"
#include "src/Tracker/PatternTracker.h"

namespace Ui {
class FusionWindow;
}

class FusionWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit FusionWindow(QWidget *parent = nullptr);

    bool init();

    ~FusionWindow() override;

    virtual void keyPressEvent(QKeyEvent *event) override;

protected:
    ObserverT m_irObserver1, m_irObserver2;
    ObserverT m_colorObserver1, m_colorObserver2;
    ObserverT m_cloudObserver1, m_cloudObserver2;
    ObserverT m_trackerObserver1, m_trackerObserver2;

    Ui::FusionWindow *ui;

    std::shared_ptr<BaseCam> m_cam = nullptr;
    std::shared_ptr<FFmpegCamRecorder> m_recorder = nullptr;
    std::shared_ptr<FFmpegCamPlayer> m_player = nullptr;

    std::shared_ptr<PatternTracker> m_patternTracker1 = nullptr, m_patternTracker2 = nullptr;
    std::shared_ptr<CloudFromDepth> m_pointsMapper1 = nullptr, m_pointsMapper2 = nullptr;

    cv::Mat m_pointCloud, m_pointCloud1, m_pointCloud2;
    cv::Mat m_pointColors, m_pointColors1, m_pointColors2;

    cv::Matx44d m_patternMat1, m_patternMat2;
    bool m_mat1_found = false;
    std::mutex m_cloudMutex, m_colorMutex;
};

#endif // SERVERWINDOW_H
