//
// Created by ascadian on 9/24/18.
//

#include "ImageStreamSession.h"
#include "src/Core/ImageStream.h"
#include "RequestStreams.h"
#include "Connection.h"

namespace dyncam {

ImageStreamSession::ImageStreamSession(boost::asio::ip::basic_endpoint<boost::asio::ip::tcp> endpoint,
                                         std::shared_ptr<ImageStream> stream) :
                                         m_stream(std::move(stream)){
    m_stream->events = m_eventSource;
    m_endpoint = endpoint;
    start(endpoint);
}

void ImageStreamSession::sendStreamRequest() {
    std::cout << "Client: Requesting stream" << std::endl;

    // bind connection info in lambda to make sure it stays in scope during async write
    std::shared_ptr<RequestStreams> reqStreamList = std::make_shared<RequestStreams>(m_stream->GetId(), m_stream->getType());
    m_connection->async_write(*reqStreamList,
                              [this, reqStreamList=reqStreamList] (const boost::system::error_code &e) {
                                  if (e) {
                                      std::cerr << "Client: " << e.message() << std::endl;
                                  } else {
                                      std::cout << "Client: RequestStreams send" << std::endl;
                                      readStream();
                                  }
                              });
}

void ImageStreamSession::readStream() {
    std::shared_ptr<cv::Mat> m = std::make_shared<cv::Mat>();
    m_connection->async_read(*m,
        [this, m](const boost::system::error_code &e) mutable {
         if (e) {
             std::cerr << "ClientSess: " << e.message() << std::endl;
         } else {
             //std::cout << "ClientSess received image: " << std::endl << m << std::endl;
             m_eventSource << *m;
             readStream();
         }
        });
}

void ImageStreamSession::on_connected() {
    ClientBase::on_connected();
    sendStreamRequest();
}

} // namespace dyncam {