//
// Created by ascadian on 11/26/17.
//

#include "TestCam.h"

#include <chrono>

namespace dyncam {
std::set<StreamTypes> TestCam::getStreamCapabilities() {
    return {StreamTypes::COLOR, StreamTypes::DEPTH, StreamTypes::INTRINSICS};
}

TestCam::~TestCam() {
    Close();
}

TestCam::TestCam() {
    // Unique id for this camera. For parent child relation in collection
    int cameraId = Stream::GenerateUniqueId();

    Streams.SetSerial("TestCam: " + std::to_string(cameraId));

    cv::Size colorSize(512, 512);
    cv::Size irSize(1024, 1024);

    colorStream_ = std::make_shared<ImageStream>(StreamTypes::COLOR, cameraId, colorSize, CV_8UC4);

    depthStream_ = std::make_shared<ImageStream>(StreamTypes::DEPTH, cameraId, irSize,    CV_32FC1);
    intrStream_  = std::make_shared<IntrinsicsStream>(cameraId);

    colorStream_->events = colorSource_;
    depthStream_->events = depthSource_;

    Streams.AddStream(colorStream_);
    Streams.AddStream(depthStream_);
    Streams.AddStream(intrStream_);
}

void TestCam::streamThread() {
    while (m_doProcessing) {

        // depth
        cv::Mat depthImg = cv::Mat::ones(depthStream_->m_resolution(), CV_32FC1);
        cv::randu(depthImg, cv::Scalar(0), cv::Scalar(1));
        depthSource_ << depthImg;

        // color
        cv::Mat colorImg;
        colorImg = cv::Mat::ones(colorStream_->m_resolution(), CV_8UC4);
        cv::randu(colorImg, cv::Scalar(0, 0, 0, 0), cv::Scalar(255, 255, 255, 255));

        colorSource_ << colorImg;

        // intrinsics
        auto newIntrinsics = CamIntrinsics(rand(), rand(), rand(), rand(), rand(), rand(), rand(), rand(), rand());
        intrStream_->Intrinsics.Set(newIntrinsics);

        std::this_thread::sleep_for(std::chrono::milliseconds(500));
    }
}

bool TestCam::Open() {
    if (m_state() == State::DEVICE_OPEN)
        return true;

    m_doProcessing.store(true);
    m_streamThread = std::thread(&TestCam::streamThread, this);
    m_state.Set(State::DEVICE_OPEN);
    return true;
}

void TestCam::Close() {
    if (m_doProcessing.load()) {
        m_doProcessing.store(false);
        m_streamThread.join();
        m_state.Set(State::DEVICE_CLOSED);
    }
}

} // namespace dyncam