//
// Created by ascadian on 8/9/17.
//
#include <opencv2/core.hpp>
#include "Intrinsics.h"


CamIntrinsics::CamIntrinsics() {
    // initialize with 0
    intrinsics_ = cv::Mat(3,3,CV_32F, cv::Scalar(0));
    distCoeffs_ = cv::Mat(1, 5, CV_32F, cv::Scalar(0));
    optimalCameraMatrix_ = cv::Mat(3,3,CV_32F, cv::Scalar(0));
	extrinsicParams_ = cv::Mat(1,22, CV_32F, cv::Scalar(0));
}

CamIntrinsics::CamIntrinsics(const std::string filename): CamIntrinsics() {
    ReadFromile(std::move(filename));
}

CamIntrinsics::CamIntrinsics(const float fx, const float fy,
                             const float cx, const float cy,
                             const float k1, const float k2, const float k3,
                             const float p1, const float p2) : CamIntrinsics() {
    intrinsics_.at<float>(0) = fx;
    intrinsics_.at<float>(4) = fy;
    intrinsics_.at<float>(2) = cx;
    intrinsics_.at<float>(5) = cy;
    intrinsics_.at<float>(8) =  1;

    distCoeffs_.at<float>(0) = k1;
    distCoeffs_.at<float>(1) = k2;
    distCoeffs_.at<float>(2) = k3;
    distCoeffs_.at<float>(3) = p1;
    distCoeffs_.at<float>(4) = p2;
}

CamIntrinsics::CamIntrinsics(const float fx, const float fy,
							 const float cx, const float cy, 
							 const float shift_d, const float shift_m,
							 const float mx_x3y0, const float mx_x0y3, const float mx_x2y1, const float mx_x1y2, const float mx_x2y0,
							 const float mx_x0y2, const float mx_x1y1, const float mx_x1y0, const float mx_x0y1, const float mx_x0y0,
							 const float my_x3y0, const float my_x0y3, const float my_x2y1, const float my_x1y2, const float my_x2y0, 
							 const float my_x0y2, const float my_x1y1, const float my_x1y0, const float my_x0y1, const float my_x0y0)
	: CamIntrinsics() // We don't need to call the base constructor. We do it anyway as we might add things there later.
{
	intrinsics_.at<float>(0) = fx;
	intrinsics_.at<float>(4) = fy;
	intrinsics_.at<float>(2) = cx;
	intrinsics_.at<float>(5) = cy;
	intrinsics_.at<float>(8) = 1;

	extrinsicParams_.at<float>(0) = shift_d;
	extrinsicParams_.at<float>(1) = shift_m;

	extrinsicParams_.at<float>(2) = mx_x3y0;
	extrinsicParams_.at<float>(3) = mx_x0y3;
	extrinsicParams_.at<float>(4) = mx_x2y1;
	extrinsicParams_.at<float>(5) = mx_x1y2;
	extrinsicParams_.at<float>(6) = mx_x2y0;
	extrinsicParams_.at<float>(7) = mx_x0y2;
	extrinsicParams_.at<float>(8) = mx_x1y1;
	extrinsicParams_.at<float>(9) = mx_x1y0;
	extrinsicParams_.at<float>(10) = mx_x0y1;
	extrinsicParams_.at<float>(11) = mx_x0y0;

	extrinsicParams_.at<float>(12) = my_x3y0;
	extrinsicParams_.at<float>(13) = my_x0y3;
	extrinsicParams_.at<float>(14) = my_x2y1;
	extrinsicParams_.at<float>(15) = my_x1y2;
	extrinsicParams_.at<float>(16) = my_x2y0;
	extrinsicParams_.at<float>(17) = my_x0y2;
	extrinsicParams_.at<float>(18) = my_x1y1;
	extrinsicParams_.at<float>(19) = my_x1y0;
	extrinsicParams_.at<float>(20) = my_x0y1;
	extrinsicParams_.at<float>(21) = my_x0y0;
}

CamIntrinsics::CamIntrinsics(cv::Mat intrinsics, cv::Mat distCoeffs, cv::Mat optimalCameraMatrix)
	: CamIntrinsics() // We don't need to call the base constructor. We do it anyway as we might add things there later.
{
	intrinsics_ = intrinsics;
	distCoeffs_ = distCoeffs;
	optimalCameraMatrix_ = optimalCameraMatrix;
}

CamIntrinsics::CamIntrinsics(cv::Mat intrinsics, cv::Mat distCoeffs, cv::Mat optimalCameraMatrix, cv::Mat extrinsicParams)
    : CamIntrinsics() // We don't need to call the base constructor. We do it anyway as we might add things there later.
{
    intrinsics_ = intrinsics;
    distCoeffs_ = distCoeffs;
    optimalCameraMatrix_ = optimalCameraMatrix;
	extrinsicParams_ = extrinsicParams;
}

const bool CamIntrinsics::operator==(const CamIntrinsics &rhs) const {
    return (cv::countNonZero(intrinsics_ != rhs.intrinsics_) == 0)
        && (cv::countNonZero(distCoeffs_ != rhs.distCoeffs_) == 0)
        && (cv::countNonZero(optimalCameraMatrix_ != rhs.optimalCameraMatrix_) == 0)
		&& (cv::countNonZero(extrinsicParams_ != rhs.extrinsicParams_) ==0);
}

const bool CamIntrinsics::operator!=(const CamIntrinsics &rhs) const {
    return !operator==(rhs);
}

// intrinsics
/// fx  0 cx
///  0 fy cy
///  0  0  1
const float CamIntrinsics::get_fx() const {
    return intrinsics_.at<float>(0);
}
const float CamIntrinsics::get_fy() const {
    return intrinsics_.at<float>(4);
}
const float CamIntrinsics::get_cx() const {
    return intrinsics_.at<float>(2);
}
const float CamIntrinsics::get_cy() const {
    return intrinsics_.at<float>(5);
}

/// distCoeffs: (k_1, k_2, p_1, p_2[, k_3[, k_4, k_5, k_6]])
const float CamIntrinsics::get_k1() const {
    return distCoeffs_.at<float>(0);
}
const float CamIntrinsics::get_k2() const {
    return distCoeffs_.at<float>(1);
}
const float CamIntrinsics::get_k3() const {
    return (distCoeffs_.cols >= 5) ? distCoeffs_.at<float>(4) : 0;
}
const float CamIntrinsics::get_p1() const {
    return distCoeffs_.at<float>(2);
}
const float CamIntrinsics::get_p2() const {
    return distCoeffs_.at<float>(3);
}

//extrinsics: (shift_d, shift_m, mx_x3y0, mx_x0y3, mx_x2y1, mx_x1y2, mx_x2y0, mx_x0y2, mx_x1y1, mx_x1y0, mx_x0y1, mx_x0y0,
//             my_x3y0, my_x0y3, my_x2y1, my_x1y2, my_x2y0, my_x0y2, my_x1y1, my_x1y0, my_x0y1, my_x0y0)
const float CamIntrinsics::get_shift_d() const {
	return extrinsicParams_.at<float>(0);
}

const float CamIntrinsics::get_shift_m() const {
	return extrinsicParams_.at<float>(1);
}

const cv::Mat CamIntrinsics::get_extrinsicMatrices() const {
	cv::Mat extrinsicMatrices = extrinsicParams_(cv::Rect(2, 0, 20, 1));
	return extrinsicMatrices;
}

const void CamIntrinsics::WriteToFile(const std::string filename) const {
    cv::FileStorage fs(filename, cv::FileStorage::WRITE);
    fs << "intrinsics" << intrinsics_;
    fs << "distCoeffs" << distCoeffs_;
    fs << "optimalCameraMatrix" << optimalCameraMatrix_;
	fs << "extrinsicParams" << extrinsicParams_;
}

void CamIntrinsics::ReadFromile(const std::string filename) {
    cv::FileStorage fs(filename, cv::FileStorage::READ);
    fs["intrinsics"] >> intrinsics_;
    fs["distCoeffs"] >> distCoeffs_;
    fs["optimalCameraMatrix"] >> optimalCameraMatrix_;
	fs["extrinsicParams"] >> extrinsicParams_;
}


