//
// Created by ascadian on 5/17/18.
//

#pragma once
#include <opencv2/opencv.hpp>
#include <cereal/types/base_class.hpp>
#include <cereal/types/polymorphic.hpp>
#include "src/Core/Serialization.h"

#include "src/ReactDomain.h"

namespace dyncam {

enum class StreamTypes : unsigned int {
    UNKNOWN,
    COLOR,
    IR,
    DEPTH,
    UNDISTORTED_DEPTH,
    REGISTERED_COLOR,
    EXTRINSICS,
    INTRINSICS,
    COMPRESSED
};


class Stream {
    friend class StreamCollection;
public:
    Stream();

protected:
    explicit Stream(StreamTypes type);

    StreamTypes stream_type_;

    int dyncamVersion_ = 1;
    int id_ = -1;
    int parent_id_ = -1;


public:
    static std::string StreamTypesToString(StreamTypes type);
    static StreamTypes StreamTypesFromString(std::string const & value);

    virtual std::string ToString()= 0;

    int GetDyncamVersion();

    int GetId();
    int GetParentId();

    static int GenerateUniqueId();

    template <class Archive>
    void serialize(Archive & ar)
    {
        ar(dyncamVersion_, id_, parent_id_, stream_type_);
    }

    StreamTypes getType();

};

} // namespace dyncam