//
// Created by ascadian on 12/6/17.
//

#include "FrameTime.h"
#include <algorithm>
#include <iostream>
#include <iomanip>

void FrameTime::start() {
    timer.start();
}

void FrameTime::update() {
    auto elapsed = timer.nsecsElapsed();
    timer.restart();
    cb.push_back(elapsed);
    updateCalls++;

    msecSinceLastPrint += elapsed;
    if (msecSinceLastPrint > printInterval) {
        msecSinceLastPrint -= printInterval;
        updateStats();
    }
}

void FrameTime::updateStats() {
    minFrametime = std::numeric_limits<qint64>::max();
    maxFrametime = std::numeric_limits<qint64>::min();
    avgFrametime = 0;
    // we could have an overflow in our avg calculation but it is unlikely and not critical
    for (qint64 t : cb) {
        minFrametime = std::min(minFrametime, t);
        maxFrametime = std::max(maxFrametime, t);
        avgFrametime += t;
    }
    avgFrametime = static_cast<long>(avgFrametime / static_cast<float>(cb.size()));
}

std::ostringstream FrameTime::formatStats() {
    std::ostringstream buf;
    buf << std::fixed << std::setprecision( 2 );
    buf << "ms/f avg:" << avgFrametime / 1000000.0f
        << " max:" << maxFrametime / 1000000.0f
        << " min:" << minFrametime / 1000000.0f
        << " frames:" << updateCalls;
    return buf;
}

void FrameTime::printStats() {
    std::cout << formatStats().str() << std::endl;
}
