//
// Created by ascadian on 8/1/17.
//

#include "pointswidget.h"
#include <QMouseEvent>
#include <QOpenGLDebugLogger>
#include <QApplication>

std::mutex PointsWidget::drawMutex;


PointsWidget::PointsWidget(QWidget *parent) :
        QOpenGLWidget(parent) {

    // sample points just to show something without camera
    mPoints.create(4, 3, CV_32F);
    mPoints.at<float>(0,0) = .0;
    mPoints.at<float>(0,1) = .0;
    mPoints.at<float>(0,2) = .0;

    mPoints.at<float>(1,0) = 10;
    mPoints.at<float>(1,1) = 0;
    mPoints.at<float>(1,2) = 0;

    mPoints.at<float>(2,0) = 10;
    mPoints.at<float>(2,1) = 10;
    mPoints.at<float>(2,2) = 0;

    mPoints.at<float>(3,0) = 10;
    mPoints.at<float>(3,1) = 10;
    mPoints.at<float>(3,2) = 5;

    mColors.create(4, 4, CV_32F);
    mColors.at<float>(0,0) = 255;
    mColors.at<float>(0,1) = 0;
    mColors.at<float>(0,2) = 0;
    mColors.at<float>(0,3) = 1;

    mColors.at<float>(1,0) = 0;
    mColors.at<float>(1,1) = 255;
    mColors.at<float>(1,2) = 0;
    mColors.at<float>(1,3) = 1;

    mColors.at<float>(2,0) = 0;
    mColors.at<float>(2,1) = 0;
    mColors.at<float>(2,2) = 255;
    mColors.at<float>(2,3) = 1;

    mColors.at<float>(3,0) = 1;
    mColors.at<float>(3,1) = 0;
    mColors.at<float>(3,2) = 0;
    mColors.at<float>(3,3) = 1;

    setFocus();
}

PointsWidget::~PointsWidget()
{
    // Make sure the context is current when deleting the texture
    // and the buffers.
    makeCurrent();
    m_vbo.destroy();
    doneCurrent();
}

void PointsWidget::mousePressEvent(QMouseEvent *e)
{
    // Save mouse press position
    mousePressPosition = QVector2D(e->localPos());
	m_lastPos = e->pos();
}

void PointsWidget::mouseReleaseEvent(QMouseEvent *e)
{

}

void PointsWidget::mouseMoveEvent(QMouseEvent* e)
{
	int dx = e->x() - m_lastPos.x();
	int dy = e->y() - m_lastPos.y();

    m_cam.turn(dx/10.0f, dy/10.0f);
//    xrot += dx;
//    yrot += dy;

//	if( e->buttons() & Qt::LeftButton )
//	{
//		setPhiRotation( m_phiRot - 0.5 * dx );
//		setThetaRotation( m_thetaRot + 0.5 * dy );
//	}
//	else if( e->buttons() & Qt::RightButton )
//	{
//		m_radius += dy / 100.f;
//		if(m_radius < 0.02f) m_radius = 0.02f;	// prevent negative or too strong zoom
//	} else if (e->buttons() & Qt::MiddleButton) {
//        //if (QApplication::keyboardModifiers() && Qt::KeyboardModifier::ShiftModifier) {
//            // FIXME: based on view dir
//            m_offset.setX(m_offset.x() + dx);
//            m_offset.setY(m_offset.y() + dy);
//        //}
//    }

	m_lastPos = e->pos();
	e->accept();
	update();
}



void PointsWidget::keyPressEvent(QKeyEvent *event) {
    QWidget::keyPressEvent(event);
    if (event->key() == Qt::Key::Key_W)
    {
        m_cam.move_forward(1/5.0f);
    }

    if (event->key() == Qt::Key::Key_S)
    {
        m_cam.move_forward(-1/5.0f);
    }

    if (event->key() == Qt::Key::Key_D)
    {
        m_cam.strafe(1/5.0f);
    }

    if (event->key() == Qt::Key::Key_A)
    {
        m_cam.strafe(-1/5.0f);
    }

    if (event->key() == Qt::Key::Key_Q)
    {
        m_cam.up(1/5.0f);
    }

    if (event->key() == Qt::Key::Key_E)
    {
        m_cam.up(-1/5.0f);
    }
}

void PointsWidget::keyReleaseEvent(QKeyEvent *event) {
    QWidget::keyReleaseEvent(event);
}



void PointsWidget::initializeGL() {

    QOpenGLContext *ctx = QOpenGLContext::currentContext();
    if (ctx->hasExtension(QByteArrayLiteral("GL_KHR_debug"))) {
        QOpenGLDebugLogger *logger = new QOpenGLDebugLogger(this);
        logger->initialize(); // initializes in the current context, i.e. ctx
        connect(logger, &QOpenGLDebugLogger::messageLogged, this, [] (const QOpenGLDebugMessage &debugMessage) {
            GLenum source = debugMessage.source();
            GLenum type = debugMessage.type();
            GLuint id = debugMessage.id();
            GLenum severity = debugMessage.severity();
            QString message = debugMessage.message();

            // ignore non-significant error/warning codes
            if(id == 131169 || id == 131185 || id == 131218 || id == 131204) return;

            std::cout << "---------------" << std::endl;
            std::cout << "Debug message (" << id << "): " <<  message.toUtf8().constData() << std::endl;

            switch (source)
            {
                case GL_DEBUG_SOURCE_API:             std::cout << "Source: API"; break;
                case GL_DEBUG_SOURCE_WINDOW_SYSTEM:   std::cout << "Source: Window System"; break;
                case GL_DEBUG_SOURCE_SHADER_COMPILER: std::cout << "Source: Shader Compiler"; break;
                case GL_DEBUG_SOURCE_THIRD_PARTY:     std::cout << "Source: Third Party"; break;
                case GL_DEBUG_SOURCE_APPLICATION:     std::cout << "Source: Application"; break;
                case GL_DEBUG_SOURCE_OTHER:           std::cout << "Source: Other"; break;
            } std::cout << std::endl;

            switch (type)
            {
                case GL_DEBUG_TYPE_ERROR:               std::cout << "Type: Error"; break;
                case GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR: std::cout << "Type: Deprecated Behaviour"; break;
                case GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR:  std::cout << "Type: Undefined Behaviour"; break;
                case GL_DEBUG_TYPE_PORTABILITY:         std::cout << "Type: Portability"; break;
                case GL_DEBUG_TYPE_PERFORMANCE:         std::cout << "Type: Performance"; break;
                case GL_DEBUG_TYPE_MARKER:              std::cout << "Type: Marker"; break;
                case GL_DEBUG_TYPE_PUSH_GROUP:          std::cout << "Type: Push Group"; break;
                case GL_DEBUG_TYPE_POP_GROUP:           std::cout << "Type: Pop Group"; break;
                case GL_DEBUG_TYPE_OTHER:               std::cout << "Type: Other"; break;
            } std::cout << std::endl;

            switch (severity)
            {
                case GL_DEBUG_SEVERITY_HIGH:         std::cout << "Severity: high"; break;
                case GL_DEBUG_SEVERITY_MEDIUM:       std::cout << "Severity: medium"; break;
                case GL_DEBUG_SEVERITY_LOW:          std::cout << "Severity: low"; break;
                case GL_DEBUG_SEVERITY_NOTIFICATION: std::cout << "Severity: notification"; break;
            } std::cout << std::endl;
            std::cout << std::endl;
        });
        logger->startLogging();
    } else {
        std::cout << "Logging is not supported by your GPU." << std::endl;
    }


    initializeOpenGLFunctions();

    float r = ((float) mBgColor.darker().red()) / 255.0f;
    float g = ((float) mBgColor.darker().green()) / 255.0f;
    float b = ((float) mBgColor.darker().blue()) / 255.0f;
    glClearColor(r, g, b, 1.0f);

    mPointsShader.addShaderFromSourceCode(QOpenGLShader::Vertex,
                                           "attribute highp vec4 posAttr;\n"
										   "attribute lowp vec4 colAttr;\n"
										   "varying lowp vec4 col;\n"
										   "uniform highp mat4 matrix;\n"
                                           "uniform lowp float pointSizeUniform;\n"
                                           "uniform lowp float vertColorBlend;\n" // 1: only vertex color, 0: only color from texture
										   "void main() {\n"
                                           //"    col = vec4(1.0,1.0,1.0,1.0);\n"
                                           //"    col = vec4(pointSizeUniform,pointSizeUniform,pointSizeUniform,pointSizeUniform);\n"
                                                   "    col = vertColorBlend * gl_Color + (1-vertColorBlend) * colAttr;\n"
                                                   //"    col = gl_Color;\n"
                                           "    float p_size = pointSizeUniform;\n"
                                           "    gl_PointSize = clamp(p_size, 3.0, 20.0); \n"
										   "    gl_Position = matrix * posAttr;\n"
										   "}\n");


    mPointsShader.addShaderFromSourceCode(QOpenGLShader::Fragment,
                                           "varying lowp vec4 col;\n"
										   "void main() {\n"
										   "   gl_FragColor = col;\n"
										   "}\n");
	
    if (!mPointsShader.link()) {
        std::cerr << mPointsShader.log().toStdString() << std::endl;
    }

    // location of shader attributes
    m_vertexLocation = mPointsShader.attributeLocation("posAttr");
    m_mvpLocation = mPointsShader.uniformLocation("matrix");
    m_colortLocation = mPointsShader.attributeLocation("colAttr");
    m_pointSizeLocation = mPointsShader.uniformLocation("pointSizeUniform");
    m_vertColorBlendLocation = mPointsShader.uniformLocation("vertColorBlend");

    // Enable depth buffer
    glEnable(GL_DEPTH_TEST);

    // Enable back face culling
    glEnable(GL_CULL_FACE);

    // variable point size
    glEnable(GL_PROGRAM_POINT_SIZE);

    // Use QBasicTimer because its faster than QTimer
    timer.start(12, this);
}


void PointsWidget::resizeGL(int width, int height) {
     // Calculate aspect ratio

    //float aspect = 424 / 512.0f;
    float aspect = width / static_cast<float>(height != 0 ? height : 1);

    // Set perspective projection
    const float zNear = 0.01, zFar = 1000.0, fov = 58.0;
    mProjection.setToIdentity();
    mProjection.perspective(fov, aspect, zNear, zFar);

    // create buffers
    m_vbo.create();
    m_vbo.setUsagePattern(QOpenGLBuffer::DynamicDraw);
    m_vbo.bind();
    m_vbo.allocate(m_vboBuffSize * sizeof(float));
    m_vbo.release();

    // init buffers
    m_updateBuffers.store(true);

    updateScene();
}


void PointsWidget::updateScene() {
    if (this->isVisible()) {
        update();
    }
}

void PointsWidget::paintGL() {
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    renderImage();
}

void PointsWidget::drawAxes()
{
    glLineWidth (2.0);

    const float origin[] = {0.0f,0.0f,0.0f};
    const float xp[] = {1.0f,0.0f,0.0f};
    const float yp[] = {0.0f,1.0f,0.0f};
    const float zp[] = {0.0f,0.0f,1.0f};

    glBegin (GL_LINES);
        glColor3f (1.0,0,0); // X axis is red.
        glVertex3fv (origin);
        glVertex3fv (xp);
    glEnd();
    glBegin (GL_LINES);
        glColor3f (0,1.0f,0); // Y axis is green.
        glVertex3fv (origin);
        glVertex3fv (yp);
    glEnd();
    glBegin (GL_LINES);
        glColor3f (0,0,1.0f); // z axis is blue.
        glVertex3fv (origin);
        glVertex3fv (zp);
    glEnd();
}


void PointsWidget::renderImage() {
    std::lock_guard<std::mutex> lock(drawMutex);
    // update buffers if needed
    updateBuffers();

    m_vbo.bind();
    mPointsShader.bind();

    // Calculate model view transformation
    QMatrix4x4 matrix;
    m_cam.getCamMat(matrix);

    // model matrix
    QMatrix4x4 modelMatrix;
    modelMatrix.setToIdentity();
    //modelMatrix.translate(0,0,.5);
    //modelMatrix.rotate(90, 1,0,0);
    mPointsShader.setUniformValue(m_mvpLocation, mProjection * matrix * modelMatrix);

    // draw axes for debugging
    mPointsShader.setUniformValue(m_vertColorBlendLocation, 1.0f);
    drawAxes();
    mPointsShader.setUniformValue(m_vertColorBlendLocation, 0.0f);

    float pointSize = 1.0f;
    mPointsShader.setUniformValue(m_pointSizeLocation, pointSize);

    // Tell OpenGL programmable pipeline how to locate vertex position data
    mPointsShader.enableAttributeArray(m_vertexLocation);
    mPointsShader.setAttributeBuffer(m_vertexLocation, GL_FLOAT, 0, 3);
    // point color data
    mPointsShader.enableAttributeArray(m_colortLocation);
    const int colorOffset = static_cast<int>(mPoints.total() * mPoints.elemSize());
    mPointsShader.setAttributeBuffer(m_colortLocation, GL_FLOAT, colorOffset, 3);

    // draw points
    glDrawArrays(GL_POINTS, 0, static_cast<GLsizei>(mPoints.rows));

    m_vbo.release();
    glFlush();
}


void PointsWidget::setPointPos(const cv::Mat &points) {
    {
        std::lock_guard<std::mutex> lock(drawMutex);
        mPoints = points.clone();
    }
    setDirty();
    updateScene();
}


void PointsWidget::setPointColors(const cv::Mat &colors) {
    {
        std::lock_guard<std::mutex> lock(drawMutex);
        mColors = colors.clone();

        if(mColors.elemSize() != sizeof(GL_FLOAT)) {
            // convert unsigned to float
            mColors.convertTo(mColors, CV_32F, 1/255.0f);
        }
    }
    setDirty();
    updateScene();
}


void PointsWidget::setDirty() {
    m_updateBuffers.store(true);
}


void PointsWidget::updateBuffers() {
    if(!m_updateBuffers.load())
        return;
    // we assume the lock is active and we are on the opengl thread
    m_updateBuffers.store(false);

    // copy points into vbo
    m_vbo.bind();

    // possible optimization:
//    //auto ptr = m_vbo.mapRange(0, m_vboBuffSize, QOpenGLBuffer::RangeInvalidateBuffer | QOpenGLBuffer::RangeWrite);
//    auto ptr = m_vbo.map(QOpenGLBuffer::WriteOnly);
//    memcpy(ptr, mPoints.data(),  mPoints.size() * sizeof(mPoints[0]));
//    m_vbo.unmap();
    assert(mPoints.isContinuous()); //http://docs.opencv.org/2.4/modules/core/doc/basic_structures.html#mat
    assert(mPoints.elemSize() == sizeof(GL_FLOAT));
    assert(mColors.isContinuous()); //http://docs.opencv.org/2.4/modules/core/doc/basic_structures.html#mat


    const int pointMemSize = static_cast<int>(mPoints.total() * mPoints.elemSize());
    m_vbo.write(0, mPoints.data, pointMemSize);

    // colors
    auto colorMemSize = static_cast<const int>(mColors.total() * mColors.elemSize());
    m_vbo.write(pointMemSize, mColors.data, colorMemSize);

    assert(m_vboBuffSize >= pointMemSize + colorMemSize);

    m_vbo.release();
}