//
// Created by ascadian on 8/1/17.
//

#pragma once

#include <mutex>
#include <atomic>
#include <QOpenGLWidget>
#include <QOpenGLFunctions_2_0>
#include <opencv2/opencv.hpp>
#include <QtGui/QOpenGLShader>
#include <QOpenGLVertexArrayObject>
#include <QOpenGLBuffer>
#include <QBasicTimer>
#include "Camera.h"

class PointsWidget : public QOpenGLWidget, protected QOpenGLFunctions_2_0 {
Q_OBJECT
public:
    explicit PointsWidget(QWidget* parent = nullptr);
    ~PointsWidget();

public
Q_SLOTS:
    void setPointPos(const cv::Mat &points);
    void setPointColors(const cv::Mat &colors);
    void setDirty();

protected:

    void initializeGL() override; /// OpenGL initialization
    void paintGL() override; /// OpenGL Rendering
    void resizeGL(int width, int height) override; /// Widget Resize Event

    void updateBuffers();

    void updateScene();

    void renderImage();

    void mousePressEvent(QMouseEvent *e) override;
    void mouseReleaseEvent(QMouseEvent *e) override;
    void mouseMoveEvent(QMouseEvent *e) override;

    void keyPressEvent(QKeyEvent *event) override;

    void keyReleaseEvent(QKeyEvent *event) override;

    // testing


private:
    // continuous nx3 float, with n=number of points
    cv::Mat mPoints;
    // continuous nx4 float, with n=number of points, rgba order
    cv::Mat mColors;

    // Background color
    QColor mBgColor{200, 200, 200};

    // camera
    QPoint m_lastPos;
    float m_thetaRot = 60;	// viewing angle theta for hemisphere navigation
    float m_phiRot = 60;		// viewing angle phi   for hemisphere navigation
    float m_radius = 3;		// viewing distance from table center/ball

    QVector3D m_offset{0,0,0};

    float xpos = 0, ypos = 0, zpos = 0, xrot = 0, yrot = 0, zrot = 0, angle=0.0;

    QVector2D mousePressPosition;

    QOpenGLBuffer m_vbo{QOpenGLBuffer::VertexBuffer};
    int m_vboBuffSize = 6078464*4; // currently fits kinect TODO: make dynamic
    QMatrix4x4 mProjection;
    QOpenGLShaderProgram mPointsShader;
    // attribute location in shader
    int m_vertexLocation;
    int m_mvpLocation;
    int m_colortLocation;
    int m_pointSizeLocation;
    int m_vertColorBlendLocation;

    static std::mutex drawMutex;
    std::atomic<bool> m_updateBuffers{false};
    QBasicTimer timer;

    Camera m_cam;

    void drawAxes();
};

